@@ -6,6 +6,7 @@
 //
 #include "ui/widgets/checkbox.h"
 
+#include "base/screen_reader_state.h"
 #include "ui/effects/ripple_animation.h"
 #include "ui/basic_click_handlers.h"
 #include "ui/ui_utility.h"
@@ -16,23 +17,6 @@
 #include <QtCore/QtMath>
 
 namespace Ui {
-namespace {
-
-TextParseOptions _checkboxOptions = {
-	TextParseMultiline, // flags
-	0, // maxw
-	0, // maxh
-	Qt::LayoutDirectionAuto, // dir
-};
-
-TextParseOptions _checkboxRichOptions = {
-	TextParseMultiline, // flags
-	0, // maxw
-	0, // maxh
-	Qt::LayoutDirectionAuto, // dir
-};
-
-} // namespace
 
 AbstractCheckView::AbstractCheckView(int duration, bool checked, Fn<void()> updateCallback)
 : _duration(duration)
@@ -514,28 +498,15 @@ Checkbox::Checkbox(
 , _text(
 		_st.style,
 		QString(),
-		_checkboxOptions,
+		kDefaultTextOptions,
 		_st.style.font->elidew) {
 	_check->setUpdateCallback([=] { update(); });
 	resizeToText();
 	setCursor(style::cur_pointer);
 	std::move(
 		text
-	) | rpl::start_with_next([=](TextWithEntities &&value) {
-		if (value.entities.empty()) {
-			setText(base::take(value.text));
-		} else {
-			_text.setMarkedText(
-				_st.style,
-				std::move(value),
-				_checkboxRichOptions);
-			accessibilityNameChanged();
-			resizeToText();
-			if (_text.hasLinks()) {
-				setMouseTracking(true);
-			}
-			update();
-		}
+	) | rpl::start_with_next([=](const TextWithEntities &value) {
+		setMarkedText(value);
 	}, lifetime());
 }
 
@@ -564,10 +535,15 @@ QRect Checkbox::checkRect() const {
 	}, size);
 }
 
-void Checkbox::setText(const QString &text, bool rich) {
-	_text.setText(_st.style, text, rich ? _checkboxRichOptions : _checkboxOptions);
+void Checkbox::setText(const QString &text) {
+	setMarkedText({ text });
+}
+
+void Checkbox::setMarkedText(const TextWithEntities &text) {
+	_text.setMarkedText(_st.style, text);
 	accessibilityNameChanged();
 	resizeToText();
+	setMouseTracking(_text.hasLinks());
 	update();
 }
 
@@ -630,6 +606,7 @@ void Checkbox::resizeToText() {
 void Checkbox::setChecked(bool checked, NotifyAboutChange notify) {
 	if (_check->checked() != checked) {
 		_check->setChecked(checked, anim::type::normal);
+		accessibilityStateChanged({ .checked = checked });
 		if (notify == NotifyAboutChange::Notify) {
 			_checkedChanges.fire_copy(checked);
 		}
@@ -917,13 +894,13 @@ void RadiobuttonGroup::setValue(int value) {
 	}
 }
 
-void RadiobuttonGroup::registerButton(Radiobutton *button) {
+void RadiobuttonGroup::registerButton(not_null<Radiobutton*> button) {
 	if (!base::contains(_buttons, button)) {
 		_buttons.push_back(button);
 	}
 }
 
-void RadiobuttonGroup::unregisterButton(Radiobutton *button) {
+void RadiobuttonGroup::unregisterButton(not_null<Radiobutton*> button) {
 	_buttons.erase(ranges::remove(_buttons, button), _buttons.end());
 }
 
@@ -969,6 +946,49 @@ Radiobutton::Radiobutton(
 	) | rpl::start_with_next([=] {
 		_group->setValue(_value);
 	}, lifetime());
+
+	trackScreenReaderState();
+}
+
+void Radiobutton::trackScreenReaderState() {
+	const auto reader = base::ScreenReaderState::Instance();
+	if (checkbox()->checked() && reader->active()) {
+		// We had a group without checked item before,
+		// so they all wrongfully have StrongFocus policy.
+		for (const auto &button : _group->_buttons) {
+			if (button != this) {
+				button->setFocusPolicy(Qt::NoFocus);
+			}
+		}
+	}
+
+	auto maybeValue = _group->value() | rpl::map([=](int v) {
+		return std::make_optional(v);
+	});
+	rpl::combine(
+		reader->activeValue(),
+		(_group->hasValue()
+			? std::move(maybeValue) | rpl::type_erased()
+			: rpl::single(
+				std::optional<int>()
+			) | rpl::then(std::move(maybeValue)))
+	) | rpl::map([=](bool screenReaderActive, std::optional<int> value) {
+		if (!screenReaderActive || !value.has_value()) {
+			return Qt::NoFocus;
+		} else if (value == _value) {
+			return Qt::StrongFocus;
+		}
+		for (const auto &button : _group->_buttons) {
+			if (button->_value == value) {
+				return Qt::NoFocus;
+			}
+		}
+		return Qt::StrongFocus;
+	}) | rpl::start_with_next([=](Qt::FocusPolicy value) {
+		if (focusPolicy() != value) {
+			setFocusPolicy(value);
+		}
+	}, lifetime());
 }
 
 void Radiobutton::handleNewGroupValue(int value) {
@@ -986,8 +1006,84 @@ void Radiobutton::handlePress() {
 	}
 }
 
+void Radiobutton::keyPressEvent(QKeyEvent *e) {
+	const auto key = e->key();
+	const auto vertical = (key == Qt::Key_Up || key == Qt::Key_Down);
+	const auto horizontal = (key == Qt::Key_Left || key == Qt::Key_Right);
+
+	if (!vertical && !horizontal) {
+		return Checkbox::keyPressEvent(e);
+	}
+
+	const auto &buttons = _group->_buttons;
+	if (buttons.size() < 2) {
+		e->ignore();
+		return;
+	}
+
+	const auto i = ranges::find(buttons, not_null(this));
+	if (i == end(buttons)) {
+		e->ignore();
+		return;
+	}
+
+	const auto currentIndex = std::distance(begin(buttons), i);
+	const auto neighbor = (currentIndex > 0)
+		? buttons[currentIndex - 1]
+		: buttons[currentIndex + 1];
+
+	const auto deltaY = std::abs(neighbor->y() - y());
+	const auto deltaX = std::abs(neighbor->x() - x());
+	const auto orientation = (deltaY > deltaX)
+		? Qt::Vertical
+		: Qt::Horizontal;
+
+	if ((orientation == Qt::Vertical && !vertical)
+		|| (orientation == Qt::Horizontal && !horizontal)) {
+		e->ignore();
+		return;
+	}
+
+	const auto step = (key == Qt::Key_Down || key == Qt::Key_Right) ? 1 : -1;
+	const auto nextIndex = currentIndex + step;
+
+	if (nextIndex >= 0 && nextIndex < buttons.size()) {
+		const auto nextButton = buttons[nextIndex];
+		const auto weak = base::make_weak(nextButton);
+		_group->setValue(nextButton->_value);
+		if (const auto strong = weak.get()) {
+			strong->setFocus(Qt::OtherFocusReason);
+		}
+	}
+}
+
 Radiobutton::~Radiobutton() {
 	_group->unregisterButton(this);
 }
 
+AccessibilityState Checkbox::accessibilityState() const {
+	return { .checkable = true, .checked = checked() };
+}
+
+bool Checkbox::isSubmitEvent(not_null<QKeyEvent*> e) const {
+	return !e->isAutoRepeat()
+		&& (e->key() == Qt::Key_Space
+			|| e->key() == Qt::Key_Return
+			|| e->key() == Qt::Key_Enter);
+}
+
+void Checkbox::keyPressEvent(QKeyEvent *e) {
+	if (!isSubmitEvent(e)) {
+		RippleButton::keyPressEvent(e);
+	}
+}
+
+void Checkbox::keyReleaseEvent(QKeyEvent *e) {
+	if (isSubmitEvent(e)) {
+		handlePress();
+	} else {
+		RippleButton::keyReleaseEvent(e);
+	}
+}
+
 } // namespace Ui
